@@ -649,7 +649,7 @@ ReplicatorSMM::send_state_request (const StateRequest* const req)
     {
         sst_state_ = SST_REQ_FAILED;
 
-        st_.set(state_uuid_, STATE_SEQNO());
+        st_.set(state_uuid_, STATE_SEQNO(), safe_to_bootstrap_);
         st_.mark_safe();
 
         if (state_() > S_CLOSING)
@@ -689,7 +689,7 @@ ReplicatorSMM::request_state_transfer (void* recv_ctx,
     sst_state_ = SST_WAIT;
     /* while waiting for state transfer to complete is a good point
      * to reset gcache, since it may involve some IO too */
-    gcache_.seqno_reset();
+    gcache_.seqno_reset(to_gu_uuid(group_uuid), group_seqno);
 
     if (sst_req_len != 0)
     {
@@ -713,7 +713,7 @@ ReplicatorSMM::request_state_transfer (void* recv_ctx,
             log_fatal << "Application state transfer failed. This is "
                       << "unrecoverable condition, restart required.";
 
-            st_.set(sst_uuid_, sst_seqno_);
+            st_.set(sst_uuid_, sst_seqno_, safe_to_bootstrap_);
             st_.mark_safe();
 
             abort();
