@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2012 Codership Oy <info@codership.com>
+// Copyright (C) 2012-2016 Codership Oy <info@codership.com>
 //
 
 #include "saved_state.hpp"
@@ -11,6 +11,7 @@
 #define __STDC_FORMAT_MACROS
 #include <inttypes.h>
 #include <sys/file.h>
+#include <fcntl.h>
 
 namespace galera
 {
@@ -54,9 +55,16 @@ SavedState::SavedState  (const std::string& file) :
 
     // We take exclusive lock on state file in order to avoid possibility
     // of two Galera replicators sharing the same state file.
-    if (flock(fileno(fs_), LOCK_EX|LOCK_NB))
+    struct flock flck;
+    flck.l_start  = 0;
+    flck.l_len    = 0;
+    flck.l_type   = F_WRLCK;
+    flck.l_whence = SEEK_SET;
+
+    if (::fcntl(fileno(fs_), F_SETLK, &flck))
     {
-        log_warn << "Could not get exclusive lock on state file: " << file;
+        log_warn << "Could not get exclusive lock on state file: " << file
+                 << ": " << ::strerror(errno);
         return;
     }
 
@@ -136,10 +144,18 @@ SavedState::~SavedState ()
 {
     if (fs_)
     {
-        if (flock(fileno(fs_), LOCK_UN) != 0)
+        // Closing file descriptor should release the lock, but still...
+        struct flock flck;
+        flck.l_start  = 0;
+        flck.l_len    = 0;
+        flck.l_type   = F_UNLCK;
+        flck.l_whence = SEEK_SET;
+
+        if (::fcntl(fileno(fs_), F_SETLK, &flck))
         {
-            log_error << "Could not unlock saved state file.";
+            log_warn << "Could not unlock state file: " << ::strerror(errno);
         }
+
         fclose(fs_);
     }
 }
