@@ -69,20 +69,22 @@ START_TEST(test_basic)
 
     wsrep_uuid_t  uuid;
     wsrep_seqno_t seqno;
+    bool safe_to_bootstrap;
 
     {
         SavedState st(fname);
 
-        st.get(uuid, seqno);
+        st.get(uuid, seqno, safe_to_bootstrap);
 
         fail_if (uuid  != WSREP_UUID_UNDEFINED);
         fail_if (seqno != WSREP_SEQNO_UNDEFINED);
+        fail_if (safe_to_bootstrap != true);
 
         gu_uuid_from_string("b2c01654-8dfe-11e1-0800-a834d641cfb5",
                             to_gu_uuid(uuid));
         seqno = 2345234LL;
 
-        st.set(uuid, seqno);
+        st.set(uuid, seqno, false);
     }
 
     {
@@ -90,11 +92,13 @@ START_TEST(test_basic)
 
         wsrep_uuid_t  u;
         wsrep_seqno_t s;
+        bool stb;
 
-        st.get(u, s);
+        st.get(u, s, stb);
 
         fail_if (u != uuid);
         fail_if (s != seqno);
+        fail_if (stb != false);
     }
 }
 END_TEST
@@ -107,28 +111,31 @@ START_TEST(test_unsafe)
 
     wsrep_uuid_t  uuid;
     wsrep_seqno_t seqno;
+    bool safe_to_bootstrap;
 
-    st.get(uuid, seqno);
+    st.get(uuid, seqno, safe_to_bootstrap);
 
     fail_if (uuid  == WSREP_UUID_UNDEFINED);
     fail_if (seqno == WSREP_SEQNO_UNDEFINED);
+    fail_if (safe_to_bootstrap == true);
 
-    st.set(uuid, WSREP_SEQNO_UNDEFINED);
+    st.set(uuid, WSREP_SEQNO_UNDEFINED, false);
 
     for (int i = 0; i < 100; ++i)
     {
         start_threads(&st);
         mark_point();
         usleep (TEST_USLEEP);
-        st.set(uuid, i); // make sure that state is not lost if set concurrently
+        st.set(uuid, i, false); // make sure that state is not lost if set concurrently
         mark_point();
         usleep (TEST_USLEEP);
         stop_threads();
         mark_point();
-        st.get(uuid, seqno);
+        st.get(uuid, seqno, safe_to_bootstrap);
 
         fail_if (uuid == WSREP_UUID_UNDEFINED);
         fail_if (seqno != i);
+        fail_if (safe_to_bootstrap != false);
     }
 
     long marks, locks, writes;
@@ -146,16 +153,18 @@ START_TEST(test_corrupt)
 {
     wsrep_uuid_t  uuid;
     wsrep_seqno_t seqno;
+    bool safe_to_bootstrap;
 
     {
         SavedState st(fname);
 
-        st.get(uuid, seqno);
+        st.get(uuid, seqno, safe_to_bootstrap);
 
         fail_if (uuid  == WSREP_UUID_UNDEFINED);
         fail_if (seqno == WSREP_SEQNO_UNDEFINED);
+        fail_if (safe_to_bootstrap == true);
 
-        st.set(uuid, WSREP_SEQNO_UNDEFINED);
+        st.set(uuid, WSREP_SEQNO_UNDEFINED, false);
     }
 
     long marks(0), locks(0), writes(0);
@@ -164,13 +173,13 @@ START_TEST(test_corrupt)
     {
         SavedState st(fname);
         // explicitly overwrite corruption mark.
-        st.set (uuid, seqno);
+        st.set (uuid, seqno, false);
 
         start_threads(&st);
         mark_point();
         usleep (TEST_USLEEP);
         st.mark_corrupt();
-        st.set (uuid, seqno); // make sure that corrupt stays
+        st.set (uuid, seqno, false); // make sure that corrupt stays
         usleep (TEST_USLEEP);
         mark_point();
         stop_threads();
@@ -178,11 +187,13 @@ START_TEST(test_corrupt)
 
         wsrep_uuid_t  u;
         wsrep_seqno_t s;
-        st.get(u, s);
+        bool stb;
+        st.get(u, s, stb);
 
         // make sure that mark_corrupt() stays
         fail_if (u != WSREP_UUID_UNDEFINED);
         fail_if (s != WSREP_SEQNO_UNDEFINED);
+        fail_if (stb != false);
 
         long m, l, w;
 
