@@ -152,7 +152,17 @@ struct gcs_conn
     gcs_seqno_t volatile join_seqno;
 
     /* sync control */
-    bool         sync_sent;
+    bool         sync_sent_;
+    bool         sync_sent() const
+    {
+        assert(gu_fifo_locked(recv_q));
+        return sync_sent_;
+    }
+    void         sync_sent(bool const val)
+    {
+        assert(gu_fifo_locked(recv_q));
+        sync_sent_ = val;
+    }
 
     /* gcs_core object */
     gcs_core_t*  core; // the context that is returned by
@@ -473,9 +483,15 @@ static inline bool
 gcs_send_sync_begin (gcs_conn_t* conn)
 {
     if (gu_unlikely(GCS_CONN_JOINED == conn->state)) {
-        if (conn->lower_limit >= conn->queue_len && !conn->sync_sent) {
+        if (conn->lower_limit >= conn->queue_len && !conn->sync_sent()) {
             // tripped lower slave queue limit, send SYNC message
-            conn->sync_sent = true;
+            conn->sync_sent(true);
+#if 0
+            gu_info ("Sending SYNC: state = %s, queue_len = %ld, "
+                     "lower_limit = %ld, sync_sent = %s",
+                     gcs_conn_state_str[conn->state], conn->queue_len,
+                     conn->lower_limit, conn->sync_sent() ? "true" : "false");
+#endif
             return true;
         }
 #if 0
@@ -483,7 +499,7 @@ gcs_send_sync_begin (gcs_conn_t* conn)
             gu_info ("Not sending SYNC: state = %s, queue_len = %ld, "
                      "lower_limit = %ld, sync_sent = %s",
                      gcs_conn_state_str[conn->state], conn->queue_len,
-                     conn->lower_limit, conn->sync_sent ? "true" : "false");
+                     conn->lower_limit, conn->sync_sent() ? "true" : "false");
         }
 #endif
     }
@@ -504,7 +520,9 @@ gcs_send_sync_end (gcs_conn_t* conn)
         ret = 0;
     }
     else {
-        conn->sync_sent = false;
+        gu_fifo_lock(conn->recv_q);
+        conn->sync_sent(false);
+        gu_fifo_release(conn->recv_q);
     }
 
     ret = gcs_check_error (ret, "Failed to send SYNC signal");
@@ -515,7 +533,11 @@ gcs_send_sync_end (gcs_conn_t* conn)
 static inline long
 gcs_send_sync (gcs_conn_t* conn)
 {
-    if (gcs_send_sync_begin (conn)) {
+    gu_fifo_lock(conn->recv_q);
+    bool const send_sync(gcs_send_sync_begin (conn));
+    gu_fifo_release(conn->recv_q);
+
+    if (send_sync) {
         return gcs_send_sync_end (conn);
     }
     else {
@@ -529,14 +551,14 @@ gcs_send_sync (gcs_conn_t* conn)
  */
 
 static bool
-gcs_shift_state (gcs_conn_t*      conn,
-                 gcs_conn_state_t new_state)
+gcs_shift_state (gcs_conn_t*      const conn,
+                 gcs_conn_state_t const new_state)
 {
     static const bool allowed [GCS_CONN_STATE_MAX][GCS_CONN_STATE_MAX] = {
        // SYNCED JOINED DONOR  JOINER PRIM   OPEN   CLOSED DESTR
         { false, true,  false, false, false, false, false, false }, // SYNCED
         { false, false, true,  true,  false, false, false, false }, // JOINED
-        { true,  true,  false, false, false, false, false, false }, // DONOR
+        { true,  true,  true,  false, false, false, false, false }, // DONOR
         { false, false, false, false, true,  false, false, false }, // JOINER
         { true,  true,  true,  true,  true,  true,  false, false }, // PRIMARY
         { true,  true,  true,  true,  true,  false, true,  false }, // OPEN
@@ -544,7 +566,7 @@ gcs_shift_state (gcs_conn_t*      conn,
         { false, false, false, false, false, false, true,  false }  // DESTROYED
     };
 
-    gcs_conn_state_t old_state = conn->state;
+    gcs_conn_state_t const old_state = conn->state;
 
     if (!allowed[new_state][old_state]) {
         if (old_state != new_state) {
@@ -555,10 +577,11 @@ gcs_shift_state (gcs_conn_t*      conn,
         return false;
     }
 
-    gu_info ("Shifting %s -> %s (TO: %lld)", gcs_conn_state_str[old_state],
-             gcs_conn_state_str[new_state], conn->global_seqno);
-
-    conn->state = new_state;
+    if (old_state != new_state) {
+        gu_info ("Shifting %s -> %s (TO: %lld)", gcs_conn_state_str[old_state],
+                 gcs_conn_state_str[new_state], conn->global_seqno);
+        conn->state = new_state;
+    }
 
     return true;
 }
@@ -732,8 +755,12 @@ gcs_become_joined (gcs_conn_t* conn)
 static void
 gcs_become_synced (gcs_conn_t* conn)
 {
-    gcs_shift_state (conn, GCS_CONN_SYNCED);
-    conn->sync_sent = false;
+    gu_fifo_lock(conn->recv_q);
+    {
+        gcs_shift_state (conn, GCS_CONN_SYNCED);
+        conn->sync_sent(false);
+    }
+    gu_fifo_release(conn->recv_q);
     gu_debug("Become synced, FC offset %ld", conn->fc_offset);
     conn->fc_offset = 0;
 }
@@ -841,7 +868,7 @@ gcs_handle_act_conf (gcs_conn_t* conn, const void* action)
             abort();
         }
 
-        conn->sync_sent = false;
+        conn->sync_sent(false);
 
         // need to wake up send monitor if it was paused during CC
         gcs_sm_continue(conn->sm);
@@ -915,19 +942,9 @@ gcs_handle_act_conf (gcs_conn_t* conn, const void* action)
     switch (conn->state) {
     case GCS_CONN_JOINED:
         /* One of the cases when the node can become SYNCED */
-    {
-        bool send_sync = false;
-
-        gu_fifo_lock(conn->recv_q);
-        {
-            send_sync = gcs_send_sync_begin(conn);
-        }
-        gu_fifo_release (conn->recv_q);
-
-        if (send_sync && (ret = gcs_send_sync_end (conn))) {
+        if ((ret = gcs_send_sync(conn)) < 0) {
             gu_warn ("CC: sending SYNC failed: %ld (%s)", ret, strerror (-ret));
         }
-    }
     break;
     case GCS_CONN_JOINER:
     case GCS_CONN_DONOR:
@@ -974,7 +991,8 @@ gcs_handle_state_change (gcs_conn_t*           conn,
 
     if (buf) {
         memcpy (buf, act->buf, act->buf_len);
-        /* initially act->buf points to internal static recv buffer. No leak here */
+        /* Initially act->buf points to internal static recv buffer.
+         * No leak here. */
         ((struct gcs_act*)act)->buf = buf;
         return 1;
     }
@@ -1018,8 +1036,15 @@ gcs_handle_actions (gcs_conn_t*          conn,
             gcs_become_joined (conn);
         break;
     case GCS_ACT_SYNC:
-        ret = gcs_handle_state_change (conn, &rcvd->act);
-        gcs_become_synced (conn);
+        if (rcvd->id < 0) {
+            gu_fifo_lock(conn->recv_q);
+            conn->sync_sent(false);
+            gu_fifo_release(conn->recv_q);
+            gcs_send_sync(conn);
+        } else {
+            ret = gcs_handle_state_change (conn, &rcvd->act);
+            gcs_become_synced (conn);
+        }
         break;
     default:
         break;
